import java.awt.*;
import java.io.*;

import javazoom.jlgui.basicplayer.*;

/**
 * MP3
 * <pre>
 * MP3演奏を制御します。
 *
 * MP3を鳴らすときの留意事項
 *
 * （１）
 * BufferedInputStreamを利用していますが、演奏中はcloseすることはできません。
 * closeしてしまうと、BasicPlayer側からの随時データを取得することができなくなります。
 * ただ、バッファ分のメモリ消費だけなので、気にする必要はないかと思います。
 * 一応stopメソッドでクローズします。
 *
 * （２）
 * 演奏が終了すると、以下のような例外が発生するようです。
 * Exception in thread "BasicPlayer" java.lang.NegativeArraySizeException: -1
 *         at javazoom.jlgui.basicplayer.BasicPlayer.run(BasicPlayer.java:698)
 *         at java.base/java.lang.Thread.run(Thread.java:828)
 * これは、BufferedInputStreamを読み切った時に起きるようです。
 * （オブジェクトからreadしているが、読み込めなかった。その後の処理で例外発生）
 * BasicPlayerの中のロジックを見直せば回避できると思いますが、
 * 現時点ではそのままとしています。
 * 影響としては、上記のログが出力されますが、その他の影響はなさそうです。
 * ご興味のある方は、ご自身でロジックを見直してみてください。
 *
 * （３）
 * BasicPlayerからログが出力されてきます。
 * 通常利用時にログが出力されると、いろいろとめんどそうなので、抑止しています。
 * 抑止の方法は以下のサイトの情報を参考とさせていただきました。
 * （参考ＵＲＬ）https://blogs.osdn.jp/2019/05/31/commons-logging.html#top
 * お礼申し上げます。
 *
 * </pre>
 */
public class KSoundMP3 {

	/** BasicController */
	private		BasicController		control		= null;

	/** パスを決めるオブジェクト */
	private		Object				obj			= null;

	/** ファイルパス */
	private		String				filePath	= null;

	/** ストリーム */
	private		BufferedInputStream	bis			= null;

	/**
	 * コンストラクタ
	 * <pre>
	 * MP3オブジェクトを生成します。
	 * </pre>
	 * @param obj パスを決めるオブジェクト
	 * @param filePath ファイルパス
	 */
	public KSoundMP3(Object obj, String filePath){

		try{

			// ログ出力を抑止
			java.util.logging.Logger.getLogger("javazoom.jlgui.basicplayer").setLevel(java.util.logging.Level.OFF);

			if(obj == null){
				obj = this;
			}

			// 引数を保持
			this.obj = obj;
			this.filePath = filePath;

		}catch(Exception ex){
			ex.printStackTrace();
			return;
		}

	} // end KSoundMP3

	/**
	 * 演奏スタート
	 */
	public void start(){

		// 別スレッドで演奏する。
		EventQueue.invokeLater​(() -> {

			try{

				// 一旦ストップする
				// （使い終わっているようなら、お掃除）
				this.stop();

				// BasicPlayerを生成
				BasicPlayer player = new BasicPlayer();
				// BasicControllerを取得
				control = (BasicController) player;

				// ストリーム取得
				InputStream is = obj.getClass().getResourceAsStream(filePath);
				bis = new BufferedInputStream(is);

				// Open file, or URL or Stream (shoutcast) to play.
				control.open(bis);

				// Start playback in a thread.
				this.control.play();

			}catch(BasicPlayerException ex){
				ex.printStackTrace();
				return;
			}

		});

	} // end start

	/**
	 * 演奏ストップ
	 */
	public void stop(){

		try{
			// 停止
			if(this.control != null){
				this.control.stop();
			}
		}catch(BasicPlayerException ex){
			ex.printStackTrace();
			return;
		}

		try{
			// ストリームをクローズ
			if(this.bis != null){
				this.bis.close();
			}
		}catch(IOException ex){
			ex.printStackTrace();
			return;
		}

	}

}

